import React__default, { useState, useEffect } from 'react';
import ResizeObserver from 'resize-observer-polyfill';

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

const useActive = () => {
  const _React$useState = useState(false),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        isActive = _React$useState2[0],
        setActive = _React$useState2[1];

  const bind = {
    onMouseDown: e => setActive(true),
    onMouseUp: e => setActive(false)
  };
  return [isActive, bind];
};

const useClickOutside = (ref, onClickOutside) => {
  const _React$useState = useState(false),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        isActive = _React$useState2[0],
        setActive = _React$useState2[1];

  const mousedown = e => {
    const current = ref.current;

    if (current !== null && !current.contains(e.target)) {
      setActive(true);
      onClickOutside(e);
    }
  };

  const mouseup = e => {
    const current = ref.current;

    if (current !== null && !current.contains(e.target)) {
      setActive(false);
    }
  };

  useEffect(() => {
    document.addEventListener('mousedown', mousedown);
    document.addEventListener('mouseup', mouseup);
    return () => {
      document.removeEventListener('mousedown', mousedown);
      document.removeEventListener('mouseup', mouseup);
    };
  }, [onClickOutside]);
  return [isActive];
};

const useFocus = () => {
  const _React$useState = useState(false),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        isFocused = _React$useState2[0],
        setFocused = _React$useState2[1];

  const bind = {
    onFocus: e => setFocused(true),
    onBlur: e => setFocused(false)
  };
  return [isFocused, bind];
};

const useHover = () => {
  const _React$useState = useState(false),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        isHovered = _React$useState2[0],
        setHovered = _React$useState2[1];

  const bind = {
    onMouseEnter: e => setHovered(true),
    onMouseLeave: e => setHovered(false)
  };
  return [isHovered, bind];
};

const useMousePosition = () => {
  const _React$useState = useState(0),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        x = _React$useState2[0],
        setX = _React$useState2[1];

  const _React$useState3 = useState(0),
        _React$useState4 = _slicedToArray(_React$useState3, 2),
        y = _React$useState4[0],
        setY = _React$useState4[1];

  const bind = {
    onMouseMove: (_ref) => {
      let _ref$nativeEvent = _ref.nativeEvent,
          offsetX = _ref$nativeEvent.offsetX,
          offsetY = _ref$nativeEvent.offsetY;
      setX(offsetX);
      setY(offsetY);
    }
  };
  return [x, y, bind];
};

const useResizeObserver = ref => {
  const _React$useState = React__default.useState(0),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        width = _React$useState2[0],
        setWidth = _React$useState2[1];

  const _React$useState3 = React__default.useState(0),
        _React$useState4 = _slicedToArray(_React$useState3, 2),
        height = _React$useState4[0],
        setHeight = _React$useState4[1];

  React__default.useLayoutEffect(() => {
    const current = ref.current;
    const resizeObserver = new ResizeObserver(entries => {
      const _entries$0$contentRec = entries[0].contentRect,
            width = _entries$0$contentRec.width,
            height = _entries$0$contentRec.height;
      setWidth(width);
      setHeight(height);
    });

    if (current !== null) {
      resizeObserver.observe(current);
    }

    return () => {
      resizeObserver.disconnect();
    };
  }, []);
  return [width, height];
};

const useTouch = () => {
  const _React$useState = useState(false),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        isTouched = _React$useState2[0],
        setTouched = _React$useState2[1];

  const bind = {
    onTouchStart: e => setTouched(true),
    onTouchEnd: e => setTouched(false)
  };
  return [isTouched, bind];
};

const useWindowResize = () => {
  const _React$useState = useState(window.innerWidth),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        width = _React$useState2[0],
        setWidth = _React$useState2[1];

  const _React$useState3 = useState(window.innerHeight),
        _React$useState4 = _slicedToArray(_React$useState3, 2),
        height = _React$useState4[0],
        setHeight = _React$useState4[1];

  const resize = () => {
    setWidth(window.innerWidth);
    setHeight(window.innerHeight);
  };

  useEffect(() => {
    window.addEventListener('resize', resize);
    return () => {
      window.removeEventListener('resize', resize);
    };
  }, []);
  return [width, height];
};

export { useActive, useClickOutside, useFocus, useHover, useMousePosition, useResizeObserver, useTouch, useWindowResize };
